# -*- coding: utf-8 -*-

"""The graphical part of a LAMMPS Energy step"""

import lammps_step
import seamm_widgets as sw
import tkinter as tk
import tkinter.ttk as ttk


class TkNPT(lammps_step.TkNVT):

    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        canvas=None,
        x=None,
        y=None,
        w=200,
        h=50
    ):
        '''Initialize a node

        Keyword arguments:
        '''

        super().__init__(
            tk_flowchart=tk_flowchart,
            node=node,
            canvas=canvas,
            x=x,
            y=y,
            w=w,
            h=h
        )

        # Overwrite the property metadata
        self.property_metadata = {}
        for item, data in lammps_step.properties.items():
            if ',' in item:
                continue
            if 'npt' in data["calculation"]:
                self.property_metadata[item] = data

    def create_dialog(
        self, title='Edit NPT dynamics parameters', calculation='npt'
    ):
        """Create the edit dialog!

        This is reasonably complicated, so a bit of description
        is in order. The superclasses NVT and NVE create the dialog
        along with the basic runtime and timestep (from NVE) and
        the setting the temperature and thermostat (NVT).

        This method adds a third frame with the pressure and barostat.

        The layout is handled in part by the NVT superclass, which
        handles the temperature frame. Our part is handled by three
        methods:

        * reset_dialog does the general layout of the large blocks
        * reset_pressure_frame handles the layout of the pressure
          section except for the detail of the actual pressure/stress
          which is handled by...
        * reset_stress_frame which does the detailed layout of their
          stress or pressure terms, depending on whether we are annealing,
          and the coupling between directions.
        """

        # Let parent classes do their thing.
        super().create_dialog(title=title, calculation=calculation)

        # Shortcut for parameters
        P = self.node.parameters

        # Pressure/stress controls -- create frame for them
        p_frame = self['pressure_frame'] = ttk.LabelFrame(
            self['frame'],
            borderwidth=4,
            relief='sunken',
            text='Pressure',
            labelanchor='n',
            padding=10
        )

        # Create the widgets
        for key in (
            'system type', 'barostat', 'Panneal', 'use_stress', 'couple',
            'nreset', 'mtk', 'modulus'
        ):
            self[key] = P[key].widget(p_frame)

        # The stress/pressure section is quite complicated due to
        # couplings, etc. so put in its own subsection
        s_frame = self['stress_frame'] = ttk.Frame(
            p_frame, borderwidth=4, relief='sunken'
        )

        # The last widgets
        for key in lammps_step.NPT_Parameters.parameters:
            if key not in self:
                if key[0] == 'S':
                    self[key] = P[key].widget(s_frame, width=8, unitswidth=6)
                else:
                    self[key] = P[key].widget(s_frame)

        # and labels for the directions, couplings, etc.
        for text in (
            'XX', 'YY', 'ZZ', 'XY', 'XZ', 'YZ', 'XX+YY', 'XX+ZZ', 'YY+ZZ',
            'XX+YY+ZZ'
        ):
            self[text] = ttk.Label(s_frame, text=text)

        self['stress'] = ttk.Label(s_frame, text="Stress:")
        self['initial stress'] = ttk.Label(s_frame, text="Initial stress:")
        self['final stress'] = ttk.Label(s_frame, text="Final stress:")
        self['damping'] = ttk.Label(s_frame, text="Damping:")

        # and adding bindings for appropriate widgets
        for key in ('system type', 'barostat'):
            self[key].combobox.bind(
                "<<ComboboxSelected>>", self.reset_pressure_frame
            )
        for key in ('Panneal', 'use_stress', 'couple'):
            self[key].combobox.bind(
                "<<ComboboxSelected>>", self.reset_stress_frame
            )

    def reset_dialog(self, widget=None):
        """Layout the widgets as needed for the current state"""

        row = super().reset_dialog()

        # Reset the trajectory frame to span 2 columns
        self['control_frame'].grid(row=0, column=0, columnspan=2)

        self['temperature_frame'].grid(
            row=row, column=0, sticky=tk.N, padx=10, pady=10
        )
        self.reset_temperature_frame()

        self['pressure_frame'].grid(
            row=row, column=1, sticky=tk.N, padx=10, pady=10
        )
        self.reset_pressure_frame()

        return 2

    def reset_pressure_frame(self, widget=None):
        """Layout the widgets for the pressure/stress control
        as needed for the current state"""

        # Get the values that control the layout
        barostat = self['barostat'].get()
        system_type = self['system type'].get()

        # Remove all the current widgets
        p_frame = self['pressure_frame']
        for slave in p_frame.grid_slaves():
            slave.grid_forget()

        row = 0

        # and place the needed ones back in
        self['system type'].grid(row=row, column=0, sticky=tk.W)
        row += 1

        self['barostat'].grid(row=row, column=0, sticky=tk.W)
        row += 1

        self['Panneal'].grid(row=row, column=0, sticky=tk.W)
        row += 1

        if system_type == 'fluid':
            sw.align_labels(
                (self['system type'], self['barostat'], self['Panneal'])
            )
        else:
            self['use_stress'].grid(row=row, column=0, sticky=tk.W)
            row += 1

            self['couple'].grid(row=row, column=0, sticky=tk.W)
            row += 1

            sw.align_labels(
                (
                    self['system type'], self['barostat'], self['Panneal'],
                    self['use_stress'], self['couple']
                )
            )

        self['stress_frame'].grid(row=row, column=0, sticky=tk.W)
        row += 1

        if barostat == 'Nose-Hoover':
            self['nreset'].grid(row=row, column=0, sticky=tk.W)
            row += 1
            self['mtk'].grid(row=row, column=0, sticky=tk.W)
            sw.align_labels((self['nreset'], self['mtk']))
        else:
            self['modulus'].grid(row=row, column=0, sticky=tk.W)

        # and lay out the pressure or stress terms
        self.reset_stress_frame()

    def reset_stress_frame(self, widget=None):
        """Layout the widgets for the pressure/stress
        as needed for the current state.

        We use labels across the top and left side of the
        table of stresses, then hide the labels and units
        of all the entries, except for the last entry in the
        row, which displays the units too.

        It is a bit tricky getting the column labels to lign
        up for the last item in the row because it is wider
        with the units displaying. The weird stuff with the
        columnspan=3 and setting the 'uniform' attribute of
        the columns does this. There might be better ways, but
        this works ... just be careful changing it :-).
        """

        # Get the values that control the layout
        system_type = self['system type'].get()
        barostat = self['barostat'].get()
        Panneal = self['Panneal'].get()
        if system_type == 'fluid':
            use_stress = 'isotropic pressure'
            couple = 'x, y and z'
        else:
            use_stress = self['use_stress'].get()
            couple = self['couple'].get()

        # Remove all the current widgets
        frame = self['stress_frame']
        for slave in frame.grid_slaves():
            slave.grid_forget()

        row = 0
        # and place the needed ones back in

        keep_orthorhombic = True

        if use_stress != 'isotropic pressure':
            # Annealing and stresses
            if couple == 'x, y and z':
                # all stresses
                self['XX+YY+ZZ'].grid(row=row, column=1)
                if not keep_orthorhombic:
                    self['XY'].grid(row=row, column=2)
                    self['XZ'].grid(row=row, column=3)
                    self['YZ'].grid(row=row, column=4)

                row += 1

                self['initial stress'].grid(row=row, column=0, sticky=tk.E)
                if keep_orthorhombic:
                    self['Sxx,initial'].grid(
                        row=row, column=1, columnspan=3, sticky=tk.W
                    )
                    self['Sxx,initial'].show('entry', 'units')
                    frame.columnconfigure(1, weight=1, uniform='b')
                    frame.columnconfigure(2, weight=1, uniform='b')
                else:
                    self['Sxx,initial'].grid(row=row, column=1)
                    self['Sxx,initial'].show('entry')

                    self['Sxy,initial'].grid(row=row, column=2)
                    self['Sxz,initial'].grid(row=row, column=3)
                    self['Syz,initial'].grid(
                        row=row, column=4, columnspan=3, sticky=tk.W
                    )

                    self['Sxy,initial'].show('entry')
                    self['Sxz,initial'].show('entry')
                    self['Syz,initial'].show('entry', 'units')

                    frame.columnconfigure(1, weight=1, uniform='b')
                    frame.columnconfigure(2, weight=1, uniform='b')
                    frame.columnconfigure(3, weight=1, uniform='b')
                    frame.columnconfigure(4, weight=1, uniform='b')
                    frame.columnconfigure(5, weight=1, uniform='b')
                    frame.columnconfigure(6, weight=1, uniform='b')
                row += 1

                if Panneal != 'no':
                    self['final stress'].grid(row=row, column=0, sticky=tk.E)
                    if keep_orthorhombic:
                        self['Sxx,final'].grid(
                            row=row, column=1, columnspan=3, sticky=tk.W
                        )
                        self['Sxx,final'].show('entry', 'units')
                    else:
                        self['Sxx,final'].grid(row=row, column=1)
                        self['Sxx,final'].show('entry')

                        self['Sxy,final'].grid(row=row, column=2)
                        self['Sxz,final'].grid(row=row, column=3)
                        self['Syz,final'].grid(
                            row=row, column=4, columnspan=3, sticky=tk.W
                        )

                        self['Sxy,final'].show('entry')
                        self['Sxz,final'].show('entry')
                        self['Syz,final'].show('entry', 'units')

                    row += 1

                if barostat == 'Nose-Hoover':
                    self['damping'].grid(row=row, column=0, sticky=tk.E)
                    if keep_orthorhombic:
                        self['Sxx damp'].grid(
                            row=row, column=1, columnspan=3, sticky=tk.W
                        )
                        self['Sxx damp'].show('entry', 'units')
                    else:
                        self['Sxx damp'].grid(row=row, column=1)
                        self['Sxx damp'].show('entry')

                        self['Sxy damp'].grid(row=row, column=2)
                        self['Sxz damp'].grid(row=row, column=3)
                        self['Syz damp'].grid(
                            row=row, column=4, columnspan=3, sticky=tk.W
                        )

                        self['Sxy damp'].show('entry')
                        self['Sxz damp'].show('entry')
                        self['Syz damp'].show('entry', 'units')
            elif couple == 'x and y':
                # couple xx and yy
                self['XX+YY'].grid(row=row, column=1)
                self['ZZ'].grid(row=row, column=2)
                if not keep_orthorhombic:
                    self['XY'].grid(row=row, column=3)
                    self['XZ'].grid(row=row, column=4)
                    self['YZ'].grid(row=row, column=5)

                row += 1

                if Panneal != 'no':
                    self['initial stress'].grid(row=row, column=0, sticky=tk.E)
                else:
                    self['stress'].grid(row=row, column=0, sticky=tk.E)
                self['Sxx,initial'].grid(row=row, column=1)

                self['Sxx,initial'].show('entry')
                if keep_orthorhombic:
                    self['Szz,initial'].grid(
                        row=row, column=2, columnspan=3, sticky=tk.W
                    )
                    self['Szz,initial'].show('entry', 'units')
                    frame.columnconfigure(1, weight=1, uniform='c')
                    frame.columnconfigure(2, weight=1, uniform='c')
                    frame.columnconfigure(3, weight=1, uniform='c')
                else:
                    self['Szz,initial'].show('entry')

                    self['Sxy,initial'].grid(row=row, column=3)
                    self['Sxz,initial'].grid(row=row, column=4)
                    self['Syz,initial'].grid(
                        row=row, column=5, columnspan=3, sticky=tk.W
                    )

                    self['Sxy,initial'].show('entry')
                    self['Sxz,initial'].show('entry')
                    self['Syz,initial'].show('entry', 'units')

                    frame.columnconfigure(1, weight=1, uniform='c')
                    frame.columnconfigure(2, weight=1, uniform='c')
                    frame.columnconfigure(3, weight=1, uniform='c')
                    frame.columnconfigure(4, weight=1, uniform='c')
                    frame.columnconfigure(5, weight=1, uniform='c')
                    frame.columnconfigure(6, weight=1, uniform='c')
                row += 1

                if Panneal != 'no':
                    self['final stress'].grid(row=row, column=0, sticky=tk.E)
                    self['Sxx,final'].grid(row=row, column=1)

                    self['Sxx,final'].show('entry')
                    if keep_orthorhombic:
                        self['Szz,final'].grid(
                            row=row, column=2, columnspan=3, sticky=tk.W
                        )
                        self['Szz,final'].show('entry', 'units')
                    else:
                        self['Szz,final'].grid(row=row, column=2)
                        self['Szz,final'].show('entry')

                        self['Sxy,final'].grid(row=row, column=3)
                        self['Sxz,final'].grid(row=row, column=4)
                        self['Syz,final'].grid(
                            row=row, column=5, columnspan=3, sticky=tk.W
                        )

                        self['Sxy,final'].show('entry')
                        self['Sxz,final'].show('entry')
                        self['Syz,final'].show('entry', 'units')

                    row += 1

                if barostat == 'Nose-Hoover':
                    self['damping'].grid(row=row, column=0, sticky=tk.E)
                    self['Sxx damp'].grid(row=row, column=1)

                    self['Sxx damp'].show('entry')
                    if keep_orthorhombic:
                        self['Szz damp'].grid(
                            row=row, column=2, columnspan=3, sticky=tk.W
                        )
                        self['Szz damp'].show('entry', 'units')
                    else:
                        self['Szz damp'].grid(row=row, column=2)
                        self['Szz damp'].show('entry')

                        self['Sxy damp'].grid(row=row, column=3)
                        self['Sxz damp'].grid(row=row, column=4)
                        self['Syz damp'].grid(
                            row=row, column=5, columnspan=3, sticky=tk.W
                        )

                        self['Sxy damp'].show('entry')
                        self['Sxz damp'].show('entry')
                        self['Syz damp'].show('entry', 'units')
            elif couple == 'x and z':
                # couple xx and zz
                self['XX+ZZ'].grid(row=row, column=1)
                self['YY'].grid(row=row, column=2)
                if not keep_orthorhombic:
                    self['XY'].grid(row=row, column=3)
                    self['XZ'].grid(row=row, column=4)
                    self['YZ'].grid(row=row, column=5)

                row += 1

                if Panneal != 'no':
                    self['initial stress'].grid(row=row, column=0, sticky=tk.E)
                else:
                    self['stress'].grid(row=row, column=0, sticky=tk.E)
                self['Sxx,initial'].grid(row=row, column=1)

                self['Sxx,initial'].show('entry')
                if keep_orthorhombic:
                    self['Syy,initial'].grid(
                        row=row, column=2, columnspan=3, sticky=tk.W
                    )
                    self['Syy,initial'].show('entry', 'units')
                    frame.columnconfigure(1, weight=1, uniform='d')
                    frame.columnconfigure(2, weight=1, uniform='d')
                    frame.columnconfigure(3, weight=1, uniform='d')
                else:
                    self['Syy,initial'].grid(row=row, column=2)
                    self['Syy,initial'].show('entry')

                    self['Sxy,initial'].grid(row=row, column=3)
                    self['Sxz,initial'].grid(row=row, column=4)
                    self['Syz,initial'].grid(
                        row=row, column=5, columnspan=3, sticky=tk.W
                    )

                    self['Sxy,initial'].show('entry')
                    self['Sxz,initial'].show('entry')
                    self['Syz,initial'].show('entry', 'units')

                    frame.columnconfigure(1, weight=1, uniform='d')
                    frame.columnconfigure(2, weight=1, uniform='d')
                    frame.columnconfigure(3, weight=1, uniform='d')
                    frame.columnconfigure(4, weight=1, uniform='d')
                    frame.columnconfigure(5, weight=1, uniform='d')
                    frame.columnconfigure(6, weight=1, uniform='d')
                row += 1

                if Panneal != 'no':
                    self['final stress'].grid(row=row, column=0, sticky=tk.E)
                    self['Sxx,final'].grid(row=row, column=1)

                    self['Sxx,final'].show('entry')
                    if keep_orthorhombic:
                        self['Syy,final'].grid(
                            row=row, column=2, columnspan=3, sticky=tk.W
                        )
                        self['Syy,final'].show('entry', 'units')
                    else:
                        self['Syy,final'].grid(row=row, column=2)
                        self['Syy,final'].show('entry')

                        self['Sxy,final'].grid(row=row, column=3)
                        self['Sxz,final'].grid(row=row, column=4)
                        self['Syz,final'].grid(
                            row=row, column=5, columnspan=3, sticky=tk.W
                        )

                        self['Sxy,final'].show('entry')
                        self['Sxz,final'].show('entry')
                        self['Syz,final'].show('entry', 'units')
                    row += 1

                if barostat == 'Nose-Hoover':
                    self['damping'].grid(row=row, column=0, sticky=tk.E)
                    self['Sxx damp'].grid(row=row, column=1)

                    self['Sxx damp'].show('entry')
                    if keep_orthorhombic:
                        self['Syy damp'].grid(
                            row=row, column=2, columnspan=3, sticky=tk.W
                        )
                        self['Syy damp'].show('entry', 'units')
                    else:
                        self['Syy damp'].grid(row=row, column=2)
                        self['Syy damp'].show('entry')

                        self['Sxy damp'].grid(row=row, column=3)
                        self['Sxz damp'].grid(row=row, column=4)
                        self['Syz damp'].grid(row=row, column=5)

                        self['Sxy damp'].show('entry')
                        self['Sxz damp'].show('entry')
                        self['Syz damp'].show('entry', 'units')
            elif couple == 'y and z':
                # couple yy and zz
                self['XX'].grid(row=row, column=1)
                self['YY+ZZ'].grid(row=row, column=2)
                if not keep_orthorhombic:
                    self['XY'].grid(row=row, column=3)
                    self['XZ'].grid(row=row, column=4)
                    self['YZ'].grid(row=row, column=5)

                row += 1

                if Panneal != 'no':
                    self['initial stress'].grid(row=row, column=0, sticky=tk.E)
                else:
                    self['stress'].grid(row=row, column=0, sticky=tk.E)
                self['Sxx,initial'].grid(row=row, column=1)

                self['Sxx,initial'].show('entry')
                if keep_orthorhombic:
                    self['Syy,initial'].grid(
                        row=row, column=2, columnspan=3, sticky=tk.W
                    )
                    self['Syy,initial'].show('entry', 'units')
                    frame.columnconfigure(1, weight=1, uniform='e')
                    frame.columnconfigure(2, weight=1, uniform='e')
                    frame.columnconfigure(3, weight=1, uniform='e')
                else:
                    self['Syy,initial'].grid(row=row, column=2)
                    self['Syy,initial'].show('entry')

                    self['Sxy,initial'].grid(row=row, column=3)
                    self['Sxz,initial'].grid(row=row, column=4)
                    self['Syz,initial'].grid(row=row, column=5)

                    self['Sxy,initial'].show('entry')
                    self['Sxz,initial'].show('entry')
                    self['Syz,initial'].show('entry', 'units')

                    frame.columnconfigure(1, weight=1, uniform='e')
                    frame.columnconfigure(2, weight=1, uniform='e')
                    frame.columnconfigure(3, weight=1, uniform='e')
                    frame.columnconfigure(4, weight=1, uniform='e')
                    frame.columnconfigure(5, weight=1, uniform='e')
                    frame.columnconfigure(6, weight=1, uniform='e')
                row += 1

                if Panneal != 'no':
                    self['final stress'].grid(row=row, column=0, sticky=tk.E)
                    self['Sxx,final'].grid(row=row, column=1)

                    self['Sxx,final'].show('entry')
                    if keep_orthorhombic:
                        self['Syy,final'].grid(
                            row=row, column=2, columnspan=3, sticky=tk.W
                        )
                        self['Syy,final'].show('entry', 'units')
                    else:
                        self['Syy,final'].grid(row=row, column=2)
                        self['Syy,final'].show('entry')

                        self['Sxy,final'].grid(row=row, column=3)
                        self['Sxz,final'].grid(row=row, column=4)
                        self['Syz,final'].grid(
                            row=row, column=5, columnspan=3, sticky=tk.W
                        )

                        self['Sxy,final'].show('entry')
                        self['Sxz,final'].show('entry')
                        self['Syz,final'].show('entry', 'units')

                    row += 1

                if barostat == 'Nose-Hoover':
                    self['damping'].grid(row=row, column=0, sticky=tk.E)
                    self['Sxx damp'].grid(row=row, column=1)

                    self['Sxx damp'].show('entry')
                    if keep_orthorhombic:
                        self['Syy damp'].grid(
                            row=row, column=2, columnspan=3, sticky=tk.W
                        )
                        self['Syy damp'].show('entry', 'units')
                    else:
                        self['Syy damp'].grid(row=row, column=2)
                        self['Syy damp'].show('entry')

                        self['Sxy damp'].grid(row=row, column=3)
                        self['Sxz damp'].grid(row=row, column=4)
                        self['Syz damp'].grid(
                            row=row, column=5, columnspan=3, sticky=tk.W
                        )

                        self['Sxy damp'].show('entry')
                        self['Sxz damp'].show('entry')
                        self['Syz damp'].show('entry', 'units')
            else:
                # if couple == 'none':
                # all stresses
                self['XX'].grid(row=row, column=1)
                self['YY'].grid(row=row, column=2)
                self['ZZ'].grid(row=row, column=3)
                if not keep_orthorhombic:
                    self['XY'].grid(row=row, column=4)
                    self['XZ'].grid(row=row, column=5)
                    self['YZ'].grid(row=row, column=6)

                row += 1

                if Panneal != 'no':
                    self['initial stress'].grid(row=row, column=0, sticky=tk.E)
                else:
                    self['stress'].grid(row=row, column=0, sticky=tk.E)
                self['Sxx,initial'].grid(row=row, column=1)
                self['Syy,initial'].grid(row=row, column=2)

                self['Sxx,initial'].show('entry')
                self['Syy,initial'].show('entry')
                if keep_orthorhombic:
                    self['Szz,initial'].grid(
                        row=row, column=3, columnspan=3, sticky=tk.W
                    )
                    self['Szz,initial'].show('entry', 'units')
                    frame.columnconfigure(1, weight=1, uniform='a')
                    frame.columnconfigure(2, weight=1, uniform='a')
                    frame.columnconfigure(3, weight=1, uniform='a')
                    frame.columnconfigure(4, weight=1, uniform='a')
                else:
                    self['Szz,initial'].grid(row=row, column=3)
                    self['Szz,initial'].show('entry')

                    self['Sxy,initial'].grid(row=row, column=4)
                    self['Sxz,initial'].grid(row=row, column=5)
                    self['Syz,initial'].grid(
                        row=row, column=6, columnspan=3, sticky=tk.W
                    )

                    self['Sxy,initial'].show('entry')
                    self['Sxz,initial'].show('entry')
                    self['Syz,initial'].show('entry', 'units')

                    frame.columnconfigure(1, weight=1, uniform='a')
                    frame.columnconfigure(2, weight=1, uniform='a')
                    frame.columnconfigure(3, weight=1, uniform='a')
                    frame.columnconfigure(4, weight=1, uniform='a')
                    frame.columnconfigure(5, weight=1, uniform='a')
                    frame.columnconfigure(6, weight=1, uniform='a')
                    frame.columnconfigure(7, weight=1, uniform='a')
                row += 1

                if Panneal != 'no':
                    self['final stress'].grid(row=row, column=0, sticky=tk.E)
                    self['Sxx,final'].grid(row=row, column=1)
                    self['Syy,final'].grid(row=row, column=2)

                    self['Sxx,final'].show('entry')
                    self['Syy,final'].show('entry')

                    if keep_orthorhombic:
                        self['Szz,final'].grid(
                            row=row, column=3, columnspan=3, sticky=tk.W
                        )
                        self['Szz,final'].show('entry', 'units')
                    else:
                        self['Szz,final'].grid(row=row, column=3)
                        self['Szz,final'].show('entry')

                        self['Sxy,final'].grid(row=row, column=4)
                        self['Sxz,final'].grid(row=row, column=5)
                        self['Syz,final'].grid(
                            row=row, column=6, columnspan=3, sticky=tk.W
                        )

                        self['Sxy,final'].show('entry')
                        self['Sxz,final'].show('entry')
                        self['Syz,final'].show('entry', 'units')
                    row += 1

                if barostat == 'Nose-Hoover':
                    self['damping'].grid(row=row, column=0, sticky=tk.E)
                    self['Sxx damp'].grid(row=row, column=1)
                    self['Syy damp'].grid(row=row, column=2)
                    self['Szz damp'].grid(row=row, column=3)

                    self['Sxx damp'].show('entry')
                    self['Syy damp'].show('entry')
                    if keep_orthorhombic:
                        self['Szz damp'].grid(
                            row=row, column=3, columnspan=3, sticky=tk.W
                        )
                        self['Szz damp'].show('entry', 'units')
                    else:
                        self['Szz damp'].grid(row=row, column=3)
                        self['Szz damp'].show('entry')

                        self['Sxy damp'].grid(row=row, column=4)
                        self['Sxz damp'].grid(row=row, column=5)
                        self['Syz damp'].grid(
                            row=row, column=6, columnspan=3, sticky=tk.W
                        )

                        self['Sxy damp'].show('entry')
                        self['Sxz damp'].show('entry')
                        self['Syz damp'].show('entry', 'units')
        else:
            widgets = []
            if Panneal != 'no':
                self['Pinitial'].label.configure(text='Initial pressure:')
            else:
                self['Pinitial'].label.configure(text='Pressure:')
            self['Pinitial'].grid(row=row, column=0, sticky=tk.W)
            widgets.append(self['Pinitial'])
            row += 1

            if Panneal != 'no':
                self['Pfinal'].grid(row=row, column=0, sticky=tk.W)
                widgets.append(self['Pfinal'])
                row += 1

            if barostat == 'Nose-Hoover':
                self['Pdamp'].grid(row=row, column=0, sticky=tk.W)
                widgets.append(self['Pdamp'])
                row += 1
            sw.align_labels(widgets)

    def handle_dialog(self, result):
        """Handle when the user clicks a button on the dialog,
        which can either be 'Cancel', 'Help' or 'OK', or they can
        close the dialog with the 'x' button == 'Cancel'"""

        if result == 'OK':
            # Shortcut for parameters
            P = self.node.parameters

            # Gather all the parameters. It maybe a bit of an overkill,
            # but it is easy.
            for key in lammps_step.NPT_Parameters.parameters:
                if key not in ('results', 'create table'):
                    P[key].set_from_widget()

        # Let base classes reap their parameters
        super().handle_dialog(result)



# -*- coding: utf-8 -*-

"""A custom script for LAMMPS"""

import seamm
import logging

logger = logging.getLogger(__name__)


class Custom(seamm.Node):

    def __init__(self, flowchart=None, title='Custom', extension=None):
        """Initialize the node"""

        logger.debug('Creating Custom step {}'.format(self))

        super().__init__(flowchart=flowchart, title=title, extension=extension)

        self.description = 'A custom script for LAMMPS'

        self.text = '# Custom script for LAMMPS (replace this!)\n\n'

    def get_input(self, extras=None):
        """Get the custom input for LAMMPS"""

        lines = []

        lines.append('')
        lines.append('#     custom scripting')
        lines.append('')
        lines.append(self.text)

        return lines



# -*- coding: utf-8 -*-

"""The graphical part of a LAMMPS Initialization step"""

import logging
import seamm
import tkinter as tk
import tkinter.ttk as ttk

logger = logging.getLogger(__name__)


class TkInitialization(seamm.TkNode):

    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        canvas=None,
        x=None,
        y=None,
        w=200,
        h=50,
        my_logger=logger
    ):
        """Setup  the LAMMPS initialization node.

        Keyword arguments:
        """

        # # Set the logging level for this module if requested
        # if 'lammps_tk_initialization_log_level' in self.options:
        #     logger.setLevel(self.options.lammps_tk_initialization_log_level)
        #     logger.critical(
        #         'Set log level to {}'.format(
        #             self.options.lammps_tk_initialization_log_level
        #         )
        #     )

        super().__init__(
            tk_flowchart=tk_flowchart,
            node=node,
            canvas=canvas,
            x=x,
            y=y,
            w=w,
            h=h,
            my_logger=my_logger
        )

    def right_click(self, event):
        """Probably need to add our dialog...
        """

        super().right_click(event)
        self.popup_menu.add_command(label="Edit..", command=self.edit)

        self.popup_menu.tk_popup(event.x_root, event.y_root, 0)

    def create_dialog(self):
        """Create the dialog!"""
        frame = super().create_dialog('Edit LAMMPS Initialization Step')

        # Shortcut for parameters
        P = self.node.parameters

        # Frame for options for all systems
        general = self['general'] = ttk.LabelFrame(
            frame,
            text='For all systems',
            relief=tk.SUNKEN,
            borderwidth=5,
            labelanchor=tk.N
        )

        for key in ('cutoff', 'shift_nonbond'):
            self[key] = P[key].widget(general)

        # Frame for the periodic system options, i.e. kspace, etc.
        periodic = self['periodic'] = ttk.LabelFrame(
            frame,
            text='For periodic systems',
            relief=tk.SUNKEN,
            borderwidth=5,
            labelanchor=tk.N
        )

        for key in (
            'tail_correction', 'kspace_method', 'kspace_accuracy',
            'kspace_smallq'
        ):
            self[key] = P[key].widget(periodic)

        self['kspace_method'].bind(
            "<<ComboboxSelected>>", self.kspace_method_cb
        )

        # Grid in the static part of the dialog

        self['general'].grid(row=0, column=0, sticky=tk.EW, pady=10)
        self['cutoff'].grid(row=0, column=0, sticky=tk.W)
        self['shift_nonbond'].grid(row=1, column=0, sticky=tk.W)

        self['periodic'].grid(row=1, column=0, sticky=tk.EW, pady=10)
        self.kspace_method_cb()

    def kspace_method_cb(self, event=None):
        """Grid the widgets into the dialog, depending on the current values
        of key variables. This provides a dyamic presentation to the user.
        """

        # Remove any widgets previously packed
        for slave in self['periodic'].grid_slaves():
            slave.grid_forget()

        row = 0
        self['kspace_method'].grid(row=row, column=0, sticky=tk.W)
        row += 1

        method = self['kspace_method'].get()
        if method != 'none':
            self['kspace_accuracy'].grid(row=row, column=0, sticky=tk.W)
            row += 1

            if 'few charged' in method or method[0] == '$':
                self['kspace_smallq'].grid(row=row, column=0, sticky=tk.W)
                row += 1

        if method[0] == '$' or 'dispersion' not in method:
            self['tail_correction'].grid(row=row, column=0, sticky=tk.W)
            row += 1



# -*- coding: utf-8 -*-

"""A single-point initialization in LAMMPS"""

import seamm_ff_util
import lammps_step
import logging
import seamm
import seamm_util
import seamm_util.printing as printing
from seamm_util.printing import FormattedText as __
import pprint

logger = logging.getLogger(__name__)
job = printing.getPrinter()
printer = printing.getPrinter('lammps')

msm_pair_styles = ['born', 'buck', '', 'lj/charmm', 'lj/cut']

thermo_variables = [
    'step', 'elapsed', 'elaplong', 'dt', 'time', 'cpu', 'tpcpu', 'spcpu',
    'cpuremain', 'part', 'timeremain', 'atoms', 'temp', 'press', 'pe', 'ke',
    'etotal', 'enthalpy', 'evdwl', 'ecoul', 'epair', 'ebond', 'eangle',
    'edihed', 'eimp', 'emol', 'elong', 'etail', 'vol', 'density', 'lx', 'ly',
    'lz', 'xlo', 'xhi', 'ylo', 'yhi', 'zlo', 'zhi', 'xy', 'xz', 'yz', 'xlat',
    'ylat', 'zlat', 'bonds', 'angles', 'dihedrals', 'impropers', 'pxx', 'pyy',
    'pzz', 'pxy', 'pxz', 'pyz', 'fmax', 'fnorm', 'nbuild', 'ndanger', 'cella',
    'cellb', 'cellc', 'cellalpha', 'cellbeta', 'cellgamma'
]


class Initialization(seamm.Node):

    def __init__(self, flowchart=None, title='Initialization', extension=None):
        """Initialize the node"""

        logger.debug('Creating Initialization {}'.format(self))

        super().__init__(flowchart=flowchart, title=title, extension=extension)

        self.description = []
        self.parameters = lammps_step.InitializationParameters()

    @property
    def header(self):
        """A printable header for this section of output"""
        return (
            'Step {}: {}'.format(
                '.'.join(str(e) for e in self._id), self.title
            )
        )

    @property
    def version(self):
        """The semantic version of this module.
        """
        return lammps_step.__version__

    @property
    def git_revision(self):
        """The git version of this module.
        """
        return lammps_step.__git_revision__

    @property
    def kspace_methods(self):
        """The list of avilable methods"""
        return list(lammps_step.kspace_methods)

    def description_text(self, P=None):
        """Return a short description of this step.

        Return a nicely formatted string describing what this step will
        do.

        Keyword arguments:
            P: a dictionary of parameter values, which may be variables
                or final values. If None, then the parameters values will
                be used as is.
        """

        if not P:
            P = self.parameters.values_to_dict()

        text = 'Initialize the calculation with a cutoff of {cutoff} Å'
        if P['shift_nonbond']:
            text += ', shifting the nonbond energies to 0 at the cutoff'
        text += '. If the system is periodic'
        if P['kspace_method'][0] == '$':
            text += " use the variable '{method}' to determine whether "
            text += "and how to accelerate the k-space summations."
        elif P['kspace_method'] == 'none':
            text += ' no k-space acceleration method will be used.'
        elif P['kspace_method'] == 'automatic':
            text += ' the best k-space acceleration method for the '
            text += ' molecular system will be chosen.'
        else:
            text += ' the {method} for k-space acceleration will be used.'

        if P['kspace_method'] != 'none':
            text += ' The accuracy goal is {kspace_accuracy}.'

        return self.header + '\n' + __(text, **P, indent=4 * ' ').__str__()

    def get_input(self, extras=None):
        """Get the input for the initialization of LAMMPS"""

        self.description = []
        self.description.append('   ' + self.header)

        P = self.parameters.current_values_to_dict(
            context=seamm.flowchart_variables._data
        )

        # Fix some things
        P['cutoff'] = P['cutoff'].to('angstrom').magnitude

        # Get the system
        system = self.get_variable('_system')

        # See what type of forcefield we have amd handle it
        ff = self.get_variable('_forcefield')
        if ff == 'OpenKIM':
            lammps_step.set_lammps_unit_system('metal')
            return self.OpenKIM_input()

        # Valence forcefield...
        ffname = ff.current_forcefield
        n_atoms = system.n_atoms()

        # And atom-type if necessary
        key = f'atom_types_{ffname}'
        if key not in system.atoms:
            smiles = system.to_smiles()
            logger.debug('Atom typing -- smiles = ' + smiles)
            ff_assigner = seamm_ff_util.FFAssigner(ff)
            atom_types = ff_assigner.assign(smiles, add_hydrogens=False)

            logger.info('Atom types: ' + ', '.join(atom_types))

            system.atoms.add_attribute(key, coltype='str')
            system.atoms[key] = atom_types

            printer.important(
                __(
                    f"Assigned the atom types for forcefield '{ffname}' to "
                    "the system",
                    indent=self.indent + '    '
                )
            )

        # Get the energy expression. This creates the charges on the
        # atoms as a side-effect.
        eex = ff.energy_expression(system, style='LAMMPS')
        logger.debug('energy expression:\n' + pprint.pformat(eex))

        # Determine if we have any charges, and if so, if they are sparse
        key = f'charges_{ffname}'
        if key in system.atoms:
            charges = [*system.atoms[key]]
            n_charged_atoms = 0
            smallq = float(P['kspace_smallq'])
            for charge in charges:
                if abs(charge) > smallq:
                    n_charged_atoms += 1
            fraction_charged_atoms = n_charged_atoms / n_atoms
        else:
            n_charged_atoms = 0

        lines = []
        lines.append('')
        lines.append('#     initialization of LAMMPS')
        lines.append('')
        lines.append('units               real')

        periodicity = system.periodicity
        if periodicity == 0:
            lines.append('boundary            s s s')
            tail_correction = 'no'
            string = 'Setup for a molecular (non-periodic) system.'
        elif periodicity == 3:
            lines.append('boundary            p p p')
            tail_correction = 'yes' if P['tail_correction'] and \
                              not P['shift_nonbond'] else 'no'
            string = 'Setup for a periodic (crystalline or fluid) system.'
        else:
            raise RuntimeError(
                'The LAMMPS step can only handle 0-'
                ' or 3-D periodicity at the moment!'
            )

        lines.append('atom_style          full')
        lines.append('newton              on')
        if n_atoms < 20:
            # LAMMPS has problems with bins for small systems
            lines.append('neighbor            2.0 nsq')
        lines.append('')
        lines.append('#    define the style of forcefield')
        lines.append('')

        terms = ff.ff['terms']

        logging.debug(
            'LAMMPS initialization, terms = \n' + pprint.pformat(terms)
        )

        # control of nonbonds
        nonbond_term = None
        if 'pair' in terms:
            if len(terms['pair']) != 1:
                raise RuntimeError('Cannot handle multiple nonbond terms yet!')
            nonbond_term = terms['pair'][0]

        if nonbond_term is None:
            pprint.pprint(terms)
            raise RuntimeError("Cannot find nonbond term in forcefield!")

        if nonbond_term == 'nonbond(9-6)':
            pair_style_base = 'lj/class2'
            mixing = 'sixthpower'
        elif nonbond_term == 'nonbond(12-6)':
            pair_style_base = 'lj/cut'
            # What type of mixing rule?
            modifiers = ff.ff['modifiers']['nonbond(12-6)']
            mixing = ''
            for section in modifiers:
                for item in modifiers[section]:
                    if 'combination' in item:
                        if mixing == '':
                            mixing = item.split()[1]
                        if mixing != item.split()[1]:
                            raise RuntimeError(
                                'Conflicting combination rules in '
                                "nonbond(12-6) section '" + section + "'"
                            )
            if mixing == "":
                mixing = 'geometric'
        else:
            raise RuntimeError(
                "Can't handle nonbond term {} yet!".format(nonbond_term)
            )

        shift = 'yes' if P['shift_nonbond'] else 'no'
        if P['kspace_method'] == 'automatic':
            if periodicity == 3:
                kspace_style = ''
                if n_charged_atoms == 0:
                    pair_style = pair_style_base
                    string += (
                        ' The nonbonded interactions will be evaluated using '
                        'a cutoff of {cutoff} Å. Since there are no charges '
                        'on the atoms, no long-range coulomb method will be '
                        'used.'
                    )
                else:
                    string += (
                        ' The nonbonded interactions will be evaluated using '
                        'a cutoff of {cutoff} Å, with the long-range terms '
                    )
                    pair_style = pair_style_base + '/coul/long'
                    if n_atoms < P['ewald_atom_cutoff']:
                        kspace_style = 'ewald {}'.format(P['kspace_accuracy'])
                        string += (
                            'using the Ewald summation method with '
                            'an accuracy of {kspace_accuracy}.'
                        )
                    elif fraction_charged_atoms < \
                            P['charged_atom_fraction_cutoff']:
                        kspace_style = 'pppm/cg {} {}'.format(
                            P['kspace_accuracy'],
                            P['charged_atom_fraction_cutoff']
                        )
                        string += (
                            'using the PPPM method optimized for few '
                            'atoms with charges, with '
                            'an accuracy of {kspace_accuracy}.'
                        )
                    else:
                        kspace_style = 'pppm {}'.format(P['kspace_accuracy'])
                        string += (
                            'using the PPPM method with '
                            'an accuracy of {kspace_accuracy}.'
                        )
                lines.append(
                    'pair_style          {} {}'.format(
                        pair_style, P['cutoff']
                    )
                )
                lines.append(
                    'pair_modify         mix ' + mixing +
                    ' tail {} shift {}'.format(tail_correction, shift)
                )
                if shift:
                    string += (
                        ' The van der Waals terms will be shifted '
                        'to zero energy at the cutoff distance.'
                    )
                if tail_correction:
                    string += (
                        ' A long-range correction for the '
                        'van der Waals terms will be added.'
                    )
                if kspace_style != '':
                    lines.append('kspace_style        ' + kspace_style)
            else:
                kspace_style = ''
                if n_charged_atoms == 0:
                    pair_style = pair_style_base
                    string += (
                        ' The nonbonded interactions will be evaluated using '
                        'a simple cutoff of {cutoff} Å. Since there are no '
                        'charges on the atoms, no long-range coulomb method '
                        'will be used.'
                    )
                elif (
                    n_atoms > P['msm_atom_cutoff'] and
                    pair_style_base in msm_pair_styles
                ):
                    pair_style = pair_style_base + '/coul/msm'
                    string += (
                        'The nonbonded interactions will be handled with '
                        ' a cutoff of {cutoff} Å.'
                    )
                    if fraction_charged_atoms < \
                       P['charged_atom_fraction_cutoff']:
                        kspace_style = (
                            'msm/cg {kspace_accuracy} {kspace_smallq}'
                        )
                        string += (
                            ' The MSM method will be used to handle '
                            'the longer range coulombic interactions, using '
                            'the approach tuned for systems with few charges.'
                            'The accuracy goal is {kspace_accuracy}.'
                        )
                    else:
                        kspace_style = 'msm {kspace_accuracy}'
                        string += (
                            ' The MSM method will be used to handle '
                            'the longer range coulombic interactions.'
                            'The accuracy goal is {kspace_accuracy}.'
                        )
                else:
                    pair_style = pair_style_base + '/coul/cut'
                    string += (
                        'The nonbonded interactions will be handled with '
                        ' a simple cutoff of {cutoff} Å.'
                    )
                if shift:
                    string += (
                        ' The van der Waals terms will be shifted '
                        'to zero energy at the cutoff distance.'
                    )
                lines.append(
                    'pair_style          {} {}'.format(
                        pair_style, P['cutoff']
                    )
                )
                lines.append(
                    'pair_modify         mix ' + mixing +
                    ' shift {}'.format(shift)
                )
                if kspace_style != '':
                    lines.append(
                        'kspace_style        ' + kspace_style.format(**P)
                    )
            self.description.append(__(string, indent=7 * ' ', **P))
        else:
            if periodicity == 3:
                kspace_style = ''
                if n_charged_atoms == 0 or P['kspace_style'] == 'none':
                    pair_style = pair_style_base
                elif fraction_charged_atoms < \
                        P['charged_atom_fraction_cutoff']:
                    pair_style = pair_style_base + '/coul/long'
                    kspace_style = (
                        lammps_step.kspace_methods[P['kspace_method']].format(
                            **P
                        )
                    )
                lines.append(
                    'pair_style          {} {}'.format(
                        pair_style, P['cutoff']
                    )
                )
                lines.append(
                    'pair_modify         mix ' + mixing +
                    ' tail {} shift {}'.format(tail_correction, shift)
                )
                if kspace_style != '':
                    lines.append('kspace_style        ' + kspace_style)
            else:
                if n_charged_atoms == 0:
                    pair_style = pair_style_base
                else:
                    pair_style = pair_style_base + '/coul/cut'
                lines.append(
                    'pair_style          {} {}'.format(
                        pair_style, P['cutoff']
                    )
                )
                lines.append(
                    'pair_modify         mix ' + mixing +
                    ' shift {}'.format(shift)
                )
                if 'msm' in lammps_step.kspace_methods[P['kspace_method']]:
                    kspace_style = (
                        lammps_step.kspace_methods[P['kspace_method']].format(
                            **P
                        )
                    )
                    lines.append('kspace_style        ' + kspace_style)

        if 'bond' in terms and eex['n_bonds'] > 0:
            if len(terms['bond']) == 1:
                bond_style = lammps_step.bond_style[terms['bond'][0]]
                lines.append('bond_style          ' + bond_style)
            else:
                line = 'bond_style          hybrid'
                for term in terms['bond']:
                    line += ' ' + lammps_step.bond_style[term]
                lines.append(line)
        if 'angle' in terms and eex['n_angles'] > 0:
            if len(terms['angle']) == 1:
                angle_style = lammps_step.angle_style[terms['angle'][0]]
                lines.append('angle_style         ' + angle_style)
            else:
                line = 'angle_style         hybrid'
                for term in terms['angle']:
                    line += ' ' + lammps_step.angle_style[term]
                lines.append(line)
        if 'torsion' in terms and eex['n_torsions'] > 0:
            if len(terms['torsion']) == 1:
                #  yapf: disable
                dihedral_style = lammps_step.dihedral_style[terms['torsion'][0]]  # noqa: E501
                #  yapf: enable
                lines.append('dihedral_style      ' + dihedral_style)
            else:
                line = 'dihedral_style      hybrid'
                for term in terms['torsion']:
                    line += ' ' + lammps_step.dihedral_style[term]
                lines.append(line)
        if 'out-of-plane' in terms and eex['n_oops'] > 0:
            if len(terms['out-of-plane']) == 1:
                improper_style = lammps_step.improper_style[
                    terms['out-of-plane'][0]]
                lines.append('improper_style      ' + improper_style)
            else:
                line = 'improper_style      hybrid'
                for term in terms['out-of-plane']:
                    line += ' ' + lammps_step.improper_style[term]
                lines.append(line)

        lines.append('')
        if extras is not None and 'read_data' in extras and \
                extras['read_data'] is True:

            lines.append('read_data           structure.dat')

        # Set up standard variables
        for variable in thermo_variables:
            lines.append(
                'variable            {var} equal {var}'.format(var=variable)
            )

        return (lines, eex)

    def OpenKIM_input(self):
        """Create the initialization input for a calculation using OpenKIM.
        """
        # Get the system
        system = self.get_variable('_system')

        # Get the (simple) energy expression for these systems
        eex = self.OpenKIM_energy_expression()

        lines = []
        lines.append('')
        lines.append('#     initialization of LAMMPS')
        lines.append('')
        lines.append('newton              on')
        lines.append('')
        potential = self.get_variable('_OpenKIM_Potential')
        lines.append(
            f'kim_init            {potential} metal '
            'unit_conversion_mode'
        )
        lines.append('')
        periodicity = system.periodicity
        if periodicity == 0:
            lines.append('boundary            s s s')
            string = 'Setup for a molecular (non-periodic) system.'
        elif periodicity == 3:
            lines.append('boundary            p p p')
            string = 'Setup for a periodic (crystalline or fluid) system.'
        else:
            raise RuntimeError(
                'The LAMMPS step can only handle 0-'
                ' or 3-D periodicity at the moment!'
            )
        lines.append('')
        lines.append('read_data           structure.dat')
        lines.append('')
        lines.append(f'kim_interactions    {" ".join(eex["atom types"])}')

        # Set up standard variables
        for variable in thermo_variables:
            lines.append(
                'variable            {var} equal {var}'.format(var=variable)
            )

        self.description.append(__(string, indent=self.indent + 4 * ' '))

        return (lines, eex)

    def OpenKIM_energy_expression(self):
        """Create the (simple) energy expression for OpenKIM models.
        """
        eex = {}

        # Get the system
        system = self.get_variable('_system')
        atoms = system.atoms

        # The elements (1-based!) Probably not used...
        elements = atoms.symbols()
        eex['elements'] = ['']
        eex['elements'].extend(elements)

        # The periodicity & cell parameters
        periodicity = eex['periodicity'] = system.periodicity
        if periodicity == 3:
            eex['cell'] = system['cell'].cell().parameters

        result = eex['atoms'] = []
        atom_types = eex['atom types'] = []
        masses = eex['masses'] = []

        coordinates = atoms.coordinates(fractionals=False)
        for element, xyz in zip(elements, coordinates):
            if element in atom_types:
                index = atom_types.index(element) + 1
            else:
                atom_types.append(element)
                index = len(atom_types)
                masses.append(
                    (
                        seamm_util.element_data[element]['atomic weight'],
                        element
                    )
                )
            x, y, z = xyz
            result.append((x, y, z, index))

        eex['n_atoms'] = len(result)
        eex['n_atom_types'] = len(atom_types)

        return eex



# -*- coding: utf-8 -*-

"""
lammps_step
A step for running molecular dynamics using LAMMPS.
"""

# Bring up the classes so that they appear to be directly in
# the lammps_step package.

from lammps_step.lammps_units import set_lammps_unit_system  # noqa: F401
from lammps_step.lammps_units import get_lammps_unit_system  # noqa: F401
from lammps_step.lammps_units import to_lammps_units  # noqa: F401
from lammps_step.lammps_units import from_lammps_units  # noqa: F401

from lammps_step.lammps import bond_style  # noqa: F401
from lammps_step.lammps import angle_style  # noqa: F401
from lammps_step.lammps import dihedral_style  # noqa: F401
from lammps_step.lammps import improper_style  # noqa: F401

from lammps_step.lammps import LAMMPS  # noqa: F401
from lammps_step.lammps_step import LAMMPSStep  # noqa: F401
from lammps_step.tk_lammps import TkLAMMPS  # noqa: F401

from lammps_step.custom import Custom  # noqa: F401
from lammps_step.custom_step import CustomStep  # noqa: F401
from lammps_step.tk_custom import TkCustom  # noqa: F401

from lammps_step.initialization import Initialization  # noqa: F401
from lammps_step.initialization_parameters import InitializationParameters  # noqa: F401, E501
from lammps_step.initialization_parameters import kspace_methods  # noqa: F401
from lammps_step.initialization_step import InitializationStep  # noqa: F401
from lammps_step.tk_initialization import TkInitialization  # noqa: F401

from lammps_step.energy import Energy  # noqa: F401
from lammps_step.energy_parameters import EnergyParameters  # noqa: F401
from lammps_step.energy_step import EnergyStep  # noqa: F401
from lammps_step.tk_energy import TkEnergy  # noqa: F401

from lammps_step.minimization import Minimization  # noqa: F401
from lammps_step.minimization_step import MinimizationStep  # noqa: F401
from lammps_step.tk_minimization import TkMinimization  # noqa: F401

from lammps_step.velocities import Velocities  # noqa: F401
from lammps_step.velocities_parameters import VelocitiesParameters  # noqa: F401, E501
from lammps_step.velocities_step import VelocitiesStep  # noqa: F401
from lammps_step.tk_velocities import TkVelocities  # noqa: F401

from lammps_step.nve import NVE  # noqa: F401
from lammps_step.nve_parameters import NVE_Parameters  # noqa: F401
from lammps_step.nve_step import NVEStep  # noqa: F401
from lammps_step.tk_nve import TkNVE  # noqa: F401

from lammps_step.nvt import NVT  # noqa: F401
from lammps_step.nvt import thermostat_metadata  # noqa: F401
from lammps_step.nvt_parameters import NVT_Parameters  # noqa: F401
from lammps_step.nvt_step import NVTStep  # noqa: F401
from lammps_step.tk_nvt import TkNVT  # noqa: F401

from lammps_step.npt import NPT  # noqa: F401
from lammps_step.npt_parameters import NPT_Parameters  # noqa: F401
from lammps_step.npt_step import NPTStep  # noqa: F401
from lammps_step.tk_npt import TkNPT  # noqa: F401

# Handle versioneer
from ._version import get_versions
__author__ = """Paul Saxe"""
__email__ = 'psaxe@molssi.org'
versions = get_versions()
__version__ = versions['version']
__git_revision__ = versions['full-revisionid']
del get_versions, versions

properties = {
    "T":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "temperature",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "T,stderr":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "stderr of temperature",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "T,tau":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "autocorrelation time of temperature",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "T,inefficiency":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "statistical inefficiency of temperature sampling",
            "dimensionality": "scalar",
            "type": "float",
            "units": ""
        },
    "P":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "pressure",
            "dimensionality": "scalar",
            "type": "float",
            "units": "atm"
        },
    "P,stderr":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "stderr of pressure",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "P,tau":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "autocorrelation time of pressure",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "P,inefficiency":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "statistical inefficiency of pressure sampling",
            "dimensionality": "scalar",
            "type": "float",
            "units": ""
        },
    "density":
        {
            "calculation": ["npt"],
            "description": "pressure",
            "dimensionality": "scalar",
            "type": "float",
            "units": "g/ml"
        },
    "density,stderr":
        {
            "calculation": ["npt"],
            "description": "stderr of density",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "density,tau":
        {
            "calculation": ["npt"],
            "description": "autocorrelation time of density",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "density,inefficiency":
        {
            "calculation": ["npt"],
            "description": "statistical inefficiency of density sampling",
            "dimensionality": "scalar",
            "type": "float",
            "units": ""
        },
    "a":
        {
            "calculation": ["npt"],
            "description": "cell parameter 'a'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "Å"
        },
    "a,stderr":
        {
            "calculation": ["npt"],
            "description": "stderr of cell 'a'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "a,tau":
        {
            "calculation": ["npt"],
            "description": "autocorrelation time of cell 'a'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "a,inefficiency":
        {
            "calculation": ["npt"],
            "description": "statistical inefficiency of cell 'a' sampling",
            "dimensionality": "scalar",
            "type": "float",
            "units": ""
        },
    "b":
        {
            "calculation": ["npt"],
            "description": "cell parameter 'b'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "Å"
        },
    "b,stderr":
        {
            "calculation": ["npt"],
            "description": "stderr of cell 'b'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "b,tau":
        {
            "calculation": ["npt"],
            "description": "autocorrelation time of cell 'b'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "b,inefficiency":
        {
            "calculation": ["npt"],
            "description": "statistical inefficiency of cell 'b' sampling",
            "dimensionality": "scalar",
            "type": "float",
            "units": ""
        },
    "c":
        {
            "calculation": ["npt"],
            "description": "cell parameter 'c'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "Å"
        },
    "c,stderr":
        {
            "calculation": ["npt"],
            "description": "stderr of cell 'c'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "c,tau":
        {
            "calculation": ["npt"],
            "description": "autocorrelation time of cell 'c'",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "c,inefficiency":
        {
            "calculation": ["npt"],
            "description": "statistical inefficiency of cell 'c' sampling",
            "dimensionality": "scalar",
            "type": "float",
            "units": ""
        },
    "Etot":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "total energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "kcal/mol"
        },
    "Etot,stderr":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "stderr of total energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "Etot,tau":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "autocorrelation time of total energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "Etot,inefficiency":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "statistical inefficiency of total energy sampling",
            "dimensionality": "scalar",
            "type": "float",
            "units": ""
        },
    "Eke":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "kinetic energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "kcal/mol"
        },
    "Eke,stderr":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "stderr of kinetic energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "Eke,tau":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "autocorrelation time of kinetic energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "Eke,inefficiency":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description":
                "statistical inefficiency of kinetic energy sampling",
            "dimensionality":
                "scalar",
            "type":
                "float",
            "units":
                ""
        },
    "Epe":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "potential energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "kcal/mol"
        },
    "Epe,stderr":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "stderr of potential energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "Epe,tau":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "autocorrelation time of potential energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "Epe,inefficiency":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description":
                "statistical inefficiency of potential energy sampling",
            "dimensionality":
                "scalar",
            "type":
                "float",
            "units":
                ""
        },
    "Epair":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "nonbonded (vdW & electrostatic) energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "kcal/mol"
        },
    "Epair,stderr":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "stderr of nonbond energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "K"
        },
    "Epair,tau":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description": "autocorrelation time of nonbond energy",
            "dimensionality": "scalar",
            "type": "float",
            "units": "fs"
        },
    "Epair,inefficiency":
        {
            "calculation": [
                "nve",
                "nvt",
                "npt",
            ],
            "description":
                "statistical inefficiency of nonbond energy sampling",
            "dimensionality":
                "scalar",
            "type":
                "float",
            "units":
                ""
        },
}



# -*- coding: utf-8 -*-

"""Main module."""

import lammps_step


class VelocitiesStep(object):
    my_description = {
        'description': 'Set the velocities in preparation for MD',
        'group': 'Calculations',
        'name': 'Velocities'
    }

    def __init__(self, flowchart=None, gui=None):
        """Initialize this helper class, which is used by
        the application via stevedore to get information about
        and create node objects for the flowchart
        """
        pass

    def description(self):
        """Return a description of what this extension does
        """
        return VelocitiesStep.my_description

    def create_node(self, flowchart=None, **kwargs):
        """Return the new node object"""
        return lammps_step.Velocities(flowchart=flowchart, **kwargs)

    def create_tk_node(self, canvas=None, **kwargs):
        """Return the graphical Tk node object"""
        return lammps_step.TkVelocities(canvas=canvas, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `lammps_step` package, lammps_units module."""

import lammps_step
# Need pint for formats.
from seamm_util import ureg, Q_, units_class  # noqa: F401


def test_default_system():
    """Testing that we can convert to/from the default unit system."""
    value = '2.0 kJ/mol'
    converted = lammps_step.to_lammps_units(value)
    assert (
        f'{value} in real units is {converted:.4}' ==
        '2.0 kJ/mol in real units is 0.478'
    )
    assert (
        f'{converted:.4} --> {lammps_step.from_lammps_units(converted, "kJ/mol"):~.4}'  # noqa: E501
        == '0.478 --> 2.0 kJ / mol'
    )


def test_metal_system():
    """Testing that we can convert to/from the metal unit system."""
    lammps_step.set_lammps_unit_system('metal')
    value = '2.0 kJ/mol'
    converted = lammps_step.to_lammps_units(value)
    assert (
        f'{value} in metal units is {converted:.4}' ==
        '2.0 kJ/mol in metal units is 0.02073'
    )
    assert (
        f'{converted:.4} --> {lammps_step.from_lammps_units(converted, "kJ/mol"):~.4}'  # noqa: E501
        == '0.02073 --> 2.0 kJ / mol'
    )



# -*- coding: utf-8 -*-

"""The graphical part of a LAMMPS NVT dynamics step"""

import lammps_step
import logging
import seamm_widgets as sw
import pprint  # nopep8
import tkinter as tk
import tkinter.ttk as ttk

logger = logging.getLogger(__name__)


class TkNVT(lammps_step.TkNVE):

    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        canvas=None,
        x=None,
        y=None,
        w=200,
        h=50,
        my_logger=logger
    ):
        '''Initialize a node

        Keyword arguments:
        '''

        # Set the logging level for this module if requested
        # if 'lammps_tk_nvt_log_level' in self.options:
        #     logger.setLevel(self.options.lammps_tk_nvt_log_level)
        #     logger.critical(
        #         'Set log level to {}'.format(
        #             self.options.lammps_tk_nvt_log_level
        #         )
        #     )

        # Call the constructor for the superclass
        super().__init__(
            tk_flowchart=tk_flowchart,
            node=node,
            canvas=canvas,
            x=x,
            y=y,
            w=w,
            h=h,
            my_logger=my_logger
        )

        # Overwrite the property metadata
        self.property_metadata = {}
        for item, data in lammps_step.properties.items():
            if ',' in item:
                continue
            if 'nvt' in data["calculation"]:
                self.property_metadata[item] = data

    def create_dialog(
        self, title='Edit NVT dynamics parameters', calculation='nvt'
    ):
        """Create the edit dialog!

        This is reasonably complicated, so a bit of description
        is in order. The superclass NVE creates the dialog
        along with the basic runtime and timestep.

        This method adds a second frame for setting the temperature and
        thermostat (this is the T part of NVT).

        The layout is handled in part by the NVT superclass, which
        handles the temperature frame. Our part is handled by two
        methods:

        * reset_dialog does the general layout of the large blocks
        * reset_temperature_frame handles the layout of the temperature
          section except
        """

        logger.debug('TkNVT.create_dialog')

        # Let parent classes do their thing.
        super().create_dialog(title=title, calculation=calculation)

        # Shortcut for parameters
        P = self.node.parameters

        logger.debug('Parameters:\n{}'.format(pprint.pformat(P.to_dict())))

        # Temperature frame to isolate widgets
        t_frame = self['temperature_frame'] = ttk.LabelFrame(
            self['frame'],
            borderwidth=4,
            relief='sunken',
            text='Temperature',
            labelanchor='n',
            padding=10
        )

        for key in lammps_step.NVT_Parameters.parameters:
            self[key] = P[key].widget(t_frame)

        # and binding to change as needed
        self['thermostat'].combobox.bind(
            "<<ComboboxSelected>>", self.reset_temperature_frame
        )

    def reset_dialog(self, widget=None):
        """Layout the widgets as needed for the current state"""

        row = super().reset_dialog()

        self['temperature_frame'].grid(
            row=row, column=0, sticky=tk.EW, pady=10
        )
        row += 1
        self.reset_temperature_frame()

        return row

    def reset_temperature_frame(self, widget=None):
        """Layout the widgets in the temperature frame
        as needed for the current state"""

        thermostat = self['thermostat'].get()

        t_frame = self['temperature_frame']
        for slave in t_frame.grid_slaves():
            slave.grid_forget()

        row = 0

        # Main controls
        self['T0'].grid(row=row, column=0, columnspan=2, sticky=tk.W)
        row += 1

        self['T1'].grid(row=row, column=0, columnspan=2, sticky=tk.W)
        row += 1

        self['thermostat'].grid(row=row, column=0, columnspan=2, sticky=tk.W)
        row += 1

        sw.align_labels((self['T0'], self['T1'], self['thermostat']))

        # and controls for specific thermostats
        if thermostat != 'velocity rescaling':
            self['Tdamp'].grid(row=row, column=1, sticky=tk.W)
            row += 1

        if thermostat == 'Nose-Hoover':
            self['drag'].grid(row=row, column=1, sticky=tk.W)
            row += 1

            self['Tchain'].grid(row=row, column=1, sticky=tk.W)
            row += 1

            self['Tloop'].grid(row=row, column=1, sticky=tk.W)
            row += 1

            sw.align_labels(
                (self['Tdamp'], self['Tchain'], self['Tloop'], self['drag'])
            )
        elif thermostat == 'Berendsen':
            pass
        elif 'csvr' in thermostat or 'csld' in thermostat:
            self['seed'].grid(row=row, column=1, sticky=tk.W)
            row += 1
        elif thermostat == 'velocity rescaling':
            self['frequency'].grid(row=row, column=1, sticky=tk.W)
            row += 1

            self['window'].grid(row=row, column=1, sticky=tk.W)
            row += 1

            self['fraction'].grid(row=row, column=1, sticky=tk.W)
            row += 1

            sw.align_labels(
                (self['frequency'], self['window'], self['fraction'])
            )
        elif thermostat == 'Langevin':
            self['seed'].grid(row=row, column=1, sticky=tk.W)
            row += 1
            sw.align_labels((self['Tdamp'], self['seed']))
        else:
            raise RuntimeError(
                "Don't recognize thermostat " + "'{}'".format(thermostat)
            )

        t_frame.columnconfigure(0, minsize=50)

    def handle_dialog(self, result):
        """Handle the user cancelling, closing or OK'ing the dialog.

        If the user cancels or closes the dialog with the x button,
        revert the widgets to their previous state.

        If the user clocks 'OK', save the changes to both the control
        parameters and the results requested.
        """
        if result == 'OK':
            # Shortcut for parameters
            P = self.node.parameters

            thermostat = self['thermostat'].get()

            P['T0'].set(self['T0'].get())
            P['T1'].set(self['T1'].get())
            P['thermostat'].set(thermostat)

            if thermostat != 'velocity rescaling':
                P['Tdamp'].set(self['Tdamp'].get())

            if thermostat == 'Nose-Hoover':
                P['Tchain'].set(self['Tchain'].get())
                P['Tloop'].set(self['Tloop'].get())
                P['drag'].set(self['drag'].get())
            elif 'csvr' in thermostat or 'csld' in thermostat or \
                 thermostat == 'Langevin':
                P['seed'].set(self['seed'].get())
            elif thermostat == 'velocity rescaling':
                P['frequency'].set(self['frequency'].get())
                P['window'].set(self['window'].get())
                P['fraction'].set(self['fraction'].get())

        # Let base classes reap their parameters
        super().handle_dialog(result)



# -*- coding: utf-8 -*-
"""Control parameters for NPT (canonical) dynamics"""

import lammps_step
import logging

logger = logging.getLogger(__name__)


class NPT_Parameters(lammps_step.NVT_Parameters):
    """The control parameters for NPT dynamics in LAMMPS"""

    parameters = {
        "system type": {
            "default": 'fluid',
            "kind": "string",
            "format_string": "s",
            "enumeration": (
                'fluid',
                'solid'
            ),
            "description": "Type of system:",
            "help_text": ("Whether the system is a fluid, "
                          "which flows under shear stress, "
                          "or a solid, which can resist "
                          "small shear stresses.")
        },
        "barostat": {
            "default": 'Nose-Hoover',
            "kind": "string",
            "format_string": "s",
            "enumeration": (
                'Nose-Hoover',
                'Berendsen'
            ),
            "description": "Barostat:",
            "help_text": ("The barostat used to control the "
                          "pressure")
        },
        "Panneal": {
            "default": "no",
            "kind": "boolean",
            "format_string": "s",
            "enumeration": (
                "no",
                "yes"
            ),
            "description": "Change pressure with time",
            "help_text": ("Change the pressure linearly with time "
                          "during the run, i.e. anneal the system.")
        },
        "use_stress": {
            "default": "isotropic pressure",
            "kind": "enumeration",
            "enumeration": (
                "isotropic pressure",
                "general stress"
            ),
            "format_string": "s",
            "description": "Apply",
            "help_text": ("Specify whether to apply (isotropic) "
                          "pressure to the system or explicit stresses "
                          "for each different direction.")
        },
        "couple": {
            "default": "x, y and z",
            "kind": "enumeration",
            "enumeration": (
                "x, y and z",
                "x and y",
                "x and z",
                "y and z",
                "none"
            ),
            "format_string": "s",
            "description": "Directions to couple:",
            "help_text": ("The stress in these directions will be "
                          "averaged and the cell dilated in fixed "
                          "proportions in these directions.")
        },
        "Pinitial": {
            "default": 1.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "enumeration": tuple(),
            "description": "Initial pressure:",
            "help_text": ("The initial pressure.")
        },
        "Pfinal": {
            "default": 1.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "enumeration": tuple(),
            "description": "Final pressure:",
            "help_text": ("The final pressure.")
        },
        "Pdamp": {
            "default": 1000.0,
            "kind": "float",
            "default_units": "fs",
            "format_string": ".1f",
            "description": "Damping time:",
            "help_text": ("The damping time constant for the barostat. "
                          "Typically a value around 1000 timesteps "
                          "works well.")
        },
        "Sxx,initial": {
            "default": 1.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Sxx initial:",
            "help_text": "The initial components of the stress tensor."
        },
        "Syy,initial": {
            "default": 1.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Syy initial:",
            "help_text": "The initial components of the stress tensor."
        },
        "Szz,initial": {
            "default": 1.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Szz initial:",
            "help_text": "The initial components of the stress tensor."
        },
        "Sxy,initial": {
            "default": 0.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Sxy initial:",
            "help_text": "The initial components of the stress tensor."
        },
        "Sxz,initial": {
            "default": 0.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Sxz initial:",
            "help_text": "The initial components of the stress tensor."
        },
        "Syz,initial": {
            "default": 0.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Syz initial:",
            "help_text": "The initial components of the stress tensor."
        },
        "Sxx,final": {
            "default": 1.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Sxx final:",
            "help_text": "The final components of the stress tensor."
        },
        "Syy,final": {
            "default": 1.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Syy final:",
            "help_text": "The final components of the stress tensor."
        },
        "Szz,final": {
            "default": 1.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Szz final:",
            "help_text": "The final components of the stress tensor."
        },
        "Sxy,final": {
            "default": 0.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Sxy final:",
            "help_text": "The final components of the stress tensor."
        },
        "Sxz,final": {
            "default": 0.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Sxz final:",
            "help_text": "The final components of the stress tensor."
        },
        "Syz,final": {
            "default": 0.0,
            "kind": "float",
            "default_units": "atm",
            "format_string": ".2f",
            "description": "Syz final:",
            "help_text": "The final components of the stress tensor."
        },
        "Sxx damp": {
            "default": 1000.0,
            "kind": "float",
            "default_units": "fs",
            "format_string": ".1f",
            "description": "Sxx damping time:",
            "help_text": ("The damping time constant for the barostat. "
                          "Typically a value around 1000 timesteps "
                          "works well.")
        },
        "Syy damp": {
            "default": 1000.0,
            "kind": "float",
            "default_units": "fs",
            "format_string": ".1f",
            "description": "Syy damping time:",
            "help_text": ("The damping time constant for the barostat. "
                          "Typically a value around 1000 timesteps "
                          "works well.")
        },
        "Szz damp": {
            "default": 1000.0,
            "kind": "float",
            "default_units": "fs",
            "format_string": ".1f",
            "description": "Szz damping time:",
            "help_text": ("The damping time constant for the barostat. "
                          "Typically a value around 1000 timesteps "
                          "works well.")
        },
        "Sxy damp": {
            "default": 1000.0,
            "kind": "float",
            "default_units": "fs",
            "format_string": ".1f",
            "description": "Sxy damping time:",
            "help_text": ("The damping time constant for the barostat. "
                          "Typically a value around 1000 timesteps "
                          "works well.")
        },
        "Sxz damp": {
            "default": 1000.0,
            "kind": "float",
            "default_units": "fs",
            "format_string": ".1f",
            "description": "Sxz damping time:",
            "help_text": ("The damping time constant for the barostat. "
                          "Typically a value around 1000 timesteps "
                          "works well.")
        },
        "Syz damp": {
            "default": 1000.0,
            "kind": "float",
            "default_units": "fs",
            "format_string": ".1f",
            "description": "Syz damping time:",
            "help_text": ("The damping time constant for the barostat. "
                          "Typically a value around 1000 timesteps "
                          "works well.")
        },
        "nreset": {
            "default": "never",
            "kind": "integer",
            "default_units": None,
            "format_string": "d",
            "enumeration": ("never",),
            "description": "Frequency to reset reference cell:",
            "help_text": ("How often, in number of steps, to reset "
                          "the reference cell for the strain energy.")
        },
        "mtk": {
            "default": "yes",
            "kind": "boolean",
            "format_string": "s",
            "enumeration": (
                "no",
                "yes"
            ),
            "description": "Use corrected Hoover barostat",
            "help_text": ("Include the correction terms due to Martyna, "
                          "Tuckerman, and Klein in the equations of "
                          "motion. If not, the original Hoover barostat "
                          "is used, whose volume probability distribution "
                          "function differs from the true NPT and NPH "
                          "ensembles by a factor of 1/V. By default the "
                          "correct equations are used, but in many cases "
                          "the difference is negligible.")
        },
        "modulus": {
            "default": 50.0,
            "kind": "float",
            "default_units": "GPa",
            "format_string": ".2f",
            "enumeration": tuple(),
            "description": "Bulk modulus:",
            "help_text": ("The bulk modulus. For liquids a typical "
                          "modulus ranges from a fraction of a GPa "
                          "to a few GPa. For crystalline solids, "
                          "25-500 GPa is a reasonable range.")
        },
    }

    def __init__(self, defaults={}, data=None):
        """Initialize the instance, by default from the default
        parameters given in the class"""

        super().__init__(
            defaults={**NPT_Parameters.parameters, **defaults},
            data=data
        )



# -*- coding: utf-8 -*-

"""Definition of unit systems in LAMMPS."""
from seamm_util import ureg, Q_, units_class  # noqa: F401

lammps_unit_system = 'real'

unit_systems = {
    'real': {
        'mass': 'grams/mole',
        'distance': 'angstroms',
        'time': 'femtoseconds',
        'energy': 'kcal/mole',
        'velocity': 'angstroms/femtosecond',
        'force': 'kcal/mole/angstrom',
        'torque': 'kcal/mole',
        'temperature': 'kelvin',
        'pressure': 'atmospheres',
        'dynamic viscosity': 'poise',
        'charge': 'elementary_charge',
        'dipole': 'elementary_charge*angstroms',
        'electric field': 'volts/angstrom',
        'density': 'gram/cm^3',
    },
    'metal': {
        'mass': 'grams/mole',
        'distance': 'angstroms',
        'time': 'picoseconds',
        'energy': 'eV',
        'velocity': 'angstroms/picosecond',
        'force': 'eV/angstrom',
        'torque': 'eV',
        'temperature': 'kelvin',
        'pressure': 'bars',
        'dynamic viscosity': 'poise',
        'charge': 'elementary_charge',
        'dipole': 'elementary_charge*angstroms',
        'electric field': 'volts/angstrom',
        'density': 'gram/cm^3',
    },
    'si': {
        'mass': 'kilograms',
        'distance': 'meters',
        'time': 'seconds',
        'energy': 'joules',
        'velocity': 'meters/second',
        'force': 'newtons',
        'torque': 'newton*meters',
        'temperature': 'kelvin',
        'pressure': 'pascals',
        'dynamic viscosity': 'pascal*second',
        'charge': 'coulombs',
        'dipole': 'coulombs*meters',
        'electric field': 'volts/meter',
        'density': 'kilograms/meter^3',
    },
    'cgs': {
        'mass': 'grams',
        'distance': 'centimeters',
        'time': 'seconds',
        'energy': 'ergs',
        'velocity': 'centimeters/second',
        'force': 'dynes',
        'torque': 'dyne*centimeters',
        'temperature': 'kelvin',
        'pressure': 'dyne/cm^2',
        'dynamic viscosity': 'poise',
        'charge': 'statcoulombs',
        'dipole': 'statcoulombs*cm',
        'electric field': 'statvolt/cm',
        'density': 'grams/cm^3',
    },
    'electron': {
        'mass': 'amu',
        'distance': 'bohr',
        'time': 'femtoseconds',
        'energy': 'hartrees',
        'velocity': 'bohr/a_u_time',
        'force': 'hartrees/bohr',
        'temperature': 'kelvin',
        'pressure': 'pascals',
        'charge': 'elementary_charge',
        'dipole moment': 'debye',
        'electric field': 'volts/cm',
    },
    'micro': {
        'mass': 'picograms',
        'distance': 'micrometers',
        'time': 'microseconds',
        'energy': 'picogram*micrometer^2/microsecond^2',
        'velocity': 'micrometers/microsecond',
        'force': 'picogram*micrometer/microsecond^2',
        'torque': 'picogram*micrometer^2/microsecond^2',
        'temperature': 'kelvin',
        'pressure': 'picogram/(micrometer*microsecond^2)',
        'dynamic viscosity': 'picogram/(micrometer*microsecond)',
        'charge': 'picocoulombs',
        'dipole': 'picocoulomb*micrometer',
        'electric field': 'volt/micrometer',
        'density': 'picograms/micrometer^3',
    },
    'nano': {
        'mass': 'attograms',
        'distance': 'nanometers',
        'time': 'nanoseconds',
        'energy': 'attogram*nanometer^2/nanosecond^2',
        'velocity': 'nanometers/nanosecond',
        'force': 'attogram*nanometer/nanosecond^2',
        'torque': 'attogram*nanometer^2/nanosecond^2',
        'temperature': 'kelvin',
        'pressure': 'attogram/(nanometer*nanosecond^2)',
        'dynamic viscosity': 'attogram/(nanometer*nanosecond)',
        'charge': 'elementary_charge',
        'dipole': 'elementary_charge*nanometer',
        'electric field': 'volt/nanometer',
        'density': 'attograms/nanometer^3',
    }
}

quantity_to_dimensionality = {
    "mass": [
        "[mass] / [substance]",
        "[mass]"
    ],
    "distance": [
        "[length]"
    ],
    "time": [
        "[time]"
    ],
    "energy": [
        "[length] ** 2 * [mass] / [substance] / [time] ** 2",
        "[length] ** 2 * [mass] / [time] ** 2"
    ],
    "velocity": [
        "[length] / [time]"
    ],
    "force": [
        "[length] * [mass] / [substance] / [time] ** 2",
        "[length] * [mass] / [time] ** 2"
    ],
    "torque": [
        "[length] ** 2 * [mass] / [substance] / [time] ** 2",
        "[length] ** 2 * [mass] / [time] ** 2"
    ],
    "temperature": [
        "[temperature]"
    ],
    "pressure": [
        "[mass] / [length] / [time] ** 2"
    ],
    "dynamic viscosity": [
        "[mass] / [length] / [time]"
    ],
    "charge": [
        "[current] * [time]",
        "[length] ** 1.5 * [mass] ** 0.5 / [time]"
    ],
    "dipole": [
        "[current] * [length] * [time]",
        "[length] ** 2.5 * [mass] ** 0.5 / [time]"
    ],
    "electric field": [
        "[length] * [mass] / [current] / [time] ** 3",
        "[mass] ** 0.5 / [length] ** 0.5 / [time]"
    ],
    "density": [
        "[mass] / [length] ** 3"
    ],
    "dipole moment": [
        "[current] * [length] * [time]"
    ]
}

dimensionality_to_quantity = {
    "[mass] / [substance]": [
        "mass"
    ],
    "[mass]": [
        "mass"
    ],
    "[length]": [
        "distance"
    ],
    "[time]": [
        "time"
    ],
    "[length] ** 2 * [mass] / [substance] / [time] ** 2": [
        "energy",
        "torque"
    ],
    "[length] ** 2 * [mass] / [time] ** 2": [
        "energy",
        "torque"
    ],
    "[length] / [time]": [
        "velocity"
    ],
    "[length] * [mass] / [substance] / [time] ** 2": [
        "force"
    ],
    "[length] * [mass] / [time] ** 2": [
        "force"
    ],
    "[temperature]": [
        "temperature"
    ],
    "[mass] / [length] / [time] ** 2": [
        "pressure"
    ],
    "[mass] / [length] / [time]": [
        "dynamic viscosity"
    ],
    "[current] * [time]": [
        "charge"
    ],
    "[length] ** 1.5 * [mass] ** 0.5 / [time]": [
        "charge"
    ],
    "[current] * [length] * [time]": [
        "dipole",
        "dipole moment"
    ],
    "[length] ** 2.5 * [mass] ** 0.5 / [time]": [
        "dipole"
    ],
    "[length] * [mass] / [current] / [time] ** 3": [
        "electric field"
    ],
    "[mass] ** 0.5 / [length] ** 0.5 / [time]": [
        "electric field"
    ],
    "[mass] / [length] ** 3": [
        "density"
    ]
}


def set_lammps_unit_system(value):
    """Set the default unit system for LAMMPS."""
    global lammps_unit_system
    if value not in unit_systems:
        raise ValueError(f"unit system '{value}' not recognized!")
    lammps_unit_system = value


def get_lammps_unit_system(value):
    """Get the default unit system for LAMMPS."""
    global lammps_unit_system
    return lammps_unit_system


def to_lammps_units(value, units=None, quantity=None, unit_system=None):
    """Convert the value with units to the current LAMMPS units."""
    if not isinstance(value, units_class):
        if units is not None:
            value = Q_(value, units)
        else:
            value = Q_(value)

    if quantity is None:
        quantities = dimensionality_to_quantity[str(value.dimensionality)]
        quantity = quantities[0]

    if unit_system is None:
        unit_system = lammps_unit_system

    unit_expr = unit_systems[unit_system][quantity]
    converted = value.to(unit_expr)

    return converted.magnitude


def from_lammps_units(value, units, quantity=None, unit_system=None):
    """Convert the scalar value from LAMMPS units to requested units"""
    if quantity is None:
        dimensions = Q_(1.0, units).dimensionality
        quantities = dimensionality_to_quantity[str(dimensions)]
        quantity = quantities[0]

    if unit_system is None:
        unit_system = lammps_unit_system

    unit_expr = unit_systems[unit_system][quantity]
    converted = Q_(value, unit_expr).to(units)

    return converted


if __name__ == '__main__':  # pragma: no cover
    import pint
    import json

    for unit in ureg:
        try:
            q = ureg(unit)
        except pint.errors.UndefinedUnitError:
            q = 'unknown'
        print(f'{unit} --> {q}')
    print('')

    dimensionality = {}
    for system, units in unit_systems.items():
        print(f'{system} system of units')
        for quantity, unit_expr in units.items():
            print(f'\t{quantity} uses {unit_expr}')
            q = ureg(unit_expr)
            print(f'\t\t{q}')
            dimensions = str(q.dimensionality)
            if quantity not in dimensionality:
                dimensionality[quantity] = [dimensions]
            elif dimensions not in dimensionality[quantity]:
                dimensionality[quantity].append(dimensions)
                print(f'{system}: {quantity} <-- {unit_expr}')

    print()
    print(json.dumps(dimensionality, indent=4))

    inverse = {}
    for quantity, unit_exprs in dimensionality.items():
        for unit_expr in unit_exprs:
            if unit_expr not in inverse:
                inverse[unit_expr] = [quantity]
            elif quantity != inverse[unit_expr]:
                inverse[unit_expr].append(quantity)
                print(f'quantity {quantity} != {inverse[unit_expr]} for {unit_expr}')  # noqa: E501

    print()
    print(json.dumps(inverse, indent=4))

    print()
    value = '2.0 kJ/mol'
    converted = to_lammps_units(value)
    print(f'{value} in real units is {converted:.4}')
    print(f'{converted:.4} --> {from_lammps_units(converted, "kJ/mol"):~.4}')

    lammps_unit_system = 'metal'
    converted = to_lammps_units(value)
    print(f'{value} in metal units is {converted:.4}')
    print(f'{converted:.4} --> {from_lammps_units(converted, "kJ/mol"):~.4}')



# -*- coding: utf-8 -*-

"""The graphical part of a LAMMPS Energy step"""

import lammps_step
import logging
import seamm
import tkinter.ttk as ttk

logger = logging.getLogger(__name__)


class TkEnergy(seamm.TkNode):

    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        canvas=None,
        x=None,
        y=None,
        w=200,
        h=50,
        my_logger=logger
    ):
        '''Initialize a node

        Keyword arguments:
        '''

        self.results_widgets = []

        # Call the constructor for the energy
        super().__init__(
            tk_flowchart=tk_flowchart,
            node=node,
            canvas=canvas,
            x=x,
            y=y,
            w=w,
            h=h,
            my_logger=my_logger
        )

    def right_click(self, event):
        """Probably need to add our dialog...
        """

        super().right_click(event)
        self.popup_menu.add_command(label="Edit..", command=self.edit)

        self.popup_menu.tk_popup(event.x_root, event.y_root, 0)

    def create_dialog(
        self, title='Edit LAMMPS Energy Step', calculation='energy'
    ):
        """Create the dialog!"""

        super().create_dialog(title=title, widget='notebook', results_tab=True)

        self['message'] = ttk.Label(
            self['frame'],
            text='The LAMMPS energy step has no parameters\n'
            'All relevant parameters are set in the initialization step.'
        )
        self['message'].grid()

        self.setup_results(lammps_step.properties, calculation=calculation)



# -*- coding: utf-8 -*-
"""Control parameters for a single-point energy (SPE) in LAMMPS"""

import logging
import seamm

logger = logging.getLogger(__name__)


class EnergyParameters(seamm.Parameters):
    """The control parameters for Energy dynamics in LAMMPS"""

    parameters = {
        "results": {
            "default": {},
            "kind": "dictionary",
            "default_units": None,
            "enumeration": tuple(),
            "format_string": "",
            "description": "results",
            "help_text": ("The results to save to variables or in "
                          "tables. ")
        },
        "create tables": {
            "default": "yes",
            "kind": "boolean",
            "default_units": None,
            "enumeration": ('yes', 'no'),
            "format_string": "",
            "description": "Create tables as needed:",
            "help_text": ("Whether to create tables as needed for "
                          "results being saved into tables.")
        },
    }

    def __init__(self, defaults={}, data=None):
        """Initialize the instance, by default from the default
        parameters given in the class"""

        super().__init__(
            defaults={**EnergyParameters.parameters, **defaults},
            data=data
        )



# -*- coding: utf-8 -*-
"""Control parameters for NVE (microcanonical) dynamics"""

import lammps_step
import logging

logger = logging.getLogger(__name__)


class NVE_Parameters(lammps_step.EnergyParameters):
    """The control parameters for NVE dynamics in LAMMPS"""

    parameters = {
        "run_control": {
            "default": "Until properties converge to the requested accuracy.",
            "kind": "enumeration",
            "default_units": None,
            "format_string": "",
            "enumeration": (
                "Until properties converge to the requested accuracy.",
                "For a fixed length of simulated time."
            ),
            "description": "How long to run? ",
            "help_text": (
                "How to determine when to stop the simulation. "
                "You can give a fixed length of simulation time, "
                "e.g. 15 ps, or you can ask to run long enough to "
                "determine one or more properties to a given "
                "accuracy."
            )
        },
        "time": {
            "default": 100.0,
            "kind": "float",
            "default_units": "ps",
            "format_string": ".1f",
            "description": "Simulation time:",
            "help_text": ("The time to simulate in the dynamics run.")
        },
        "maximum_time": {
            "default": 1.0,
            "kind": "float",
            "default_units": "ns",
            "format_string": ".1f",
            "description": "Maximum simulation time:",
            "help_text": (
                "The maximum time to simulate when converging properties."
            )
        },
        "timestep": {
            "default": "normal",
            "kind": "float",
            "default_units": "fs",
            "enumeration": ('normal', 'accurate but slow', 'coarse but fast'),
            "format_string": ".1f",
            "description": "Timestep:",
            "help_text": (
                "The time step for the numerical integration in "
                "the dynamics. 1 fs is safe for most systems, "
                "except perhaps at high temperatures. For systems "
                "without hydrogen, helium, lithium or other light "
                "elements, 2-4 fs steps are reasonable. The "
                "timestep needs to be less than 1/10 the highest "
                "frequency. 10^14 Hz is a period if 10 fs, and "
                "corresponds to a frequency of 3,300 wavenumbers or "
                "a wavelength of 3 micrometers.\n"
                "You can enter a value or use the choices, which pick a "
                "reasonable timestep based on the calculation."
            )
        },
        "control_properties": {
            "default": {},
            "kind": "special",
            "widget": "seamm_widgets.PropertyTable",
            "default_units": None,
            "enumeration": tuple(),
            "format_string": "",
            "description": "Convergence properties",
            "help_text": (
                "The properties to converge when controlling the run "
                "automatically."
            )
        },
        "sampling": {
            "default": '50',
            "kind": "float",
            "default_units": "fs",
            "enumeration": ('none',),
            "format_string": ".1f",
            "description": "Sampling frequency:",
            "help_text": (
                "How often to sample the energy, temperature, "
                "etc. during the dynamics run. This controls "
                "writing to the trajectory files. Faster gives "
                "more fidelity to a point, but increases the "
                "file size and slows the calculation down."
                "\nYou can ask for no sampling, give a specific interval, "
                "or allow the system the choose for you."
            )
        },
    }

    def __init__(self, defaults={}, data=None):
        """Initialize the instance, by default from the default
        parameters given in the class"""

        super().__init__(
            defaults={**NVE_Parameters.parameters, **defaults},
            data=data
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
lammps_step
A step in SEAMM Flowchart for LAMMPS simulations
"""

import sys
from setuptools import setup, find_packages
import versioneer

short_description = __doc__.split("\n")

# from https://github.com/pytest-dev/pytest-runner#conditional-requirement
needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements_install.txt') as fd:
    requirements = fd.read()

setup(
    name='lammps_step',
    author="Paul Saxe",
    author_email='psaxe@molssi.org',
    description=short_description[1],
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/x-rst',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    license='BSD-3-Clause',
    url='https://github.com/molssi-seamm/lammps_step',

    # Which Python importable modules should be included when your package is
    # installed handled automatically by setuptools. Use 'exclude' to prevent
    # some specific subpackage(s) from being added, if needed
    packages=find_packages(include=['lammps_step']),

    # Optional include package data to ship with your package. Customize
    # MANIFEST.in if the general case does not suit your needs. Comment out
    # this line to prevent the files from being packaged with your software.
    include_package_data=True,


    # Allows `setup.py test` to work correctly with pytest
    setup_requires=[] + pytest_runner,

    # Required packages, pulls from pip if needed; do not use for Conda
    # deployment
    install_requires=requirements,

    test_suite='tests',
    # tests_require=test_requirements,

    # Valid platforms your code works on, adjust to your flavor
    platforms=['Linux',
               'Mac OS-X',
               'Unix',
               'Windows'],

    # Manual control if final package is compressible or not, set False to
    # prevent the .egg from being made
    zip_safe=False,

    keywords='lammps_step',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    entry_points={
        'org.molssi.seamm': [
            'LAMMPS = lammps_step:LAMMPSStep',
        ],
        'org.molssi.seamm.tk': [
            'LAMMPS = lammps_step:LAMMPSStep',
        ],
        'org.molssi.seamm.lammps': [
            'Custom = lammps_step:CustomStep',
            'Energy = lammps_step:EnergyStep',
            'Initialization = lammps_step:InitializationStep',
            'Minimization = lammps_step:MinimizationStep',
            'NVE = lammps_step:NVEStep',
            'NVT = lammps_step:NVTStep',
            'NPT = lammps_step:NPTStep',
            'Velocities = lammps_step:VelocitiesStep',
        ],
        'org.molssi.seamm.lammps.tk': [
            'Custom = lammps_step:CustomStep',
            'Energy = lammps_step:EnergyStep',
            'Initialization = lammps_step:InitializationStep',
            'Minimization = lammps_step:MinimizationStep',
            'NVE = lammps_step:NVEStep',
            'NVT = lammps_step:NVTStep',
            'NPT = lammps_step:NPTStep',
            'Velocities = lammps_step:VelocitiesStep',
        ],
    }
)



# -*- coding: utf-8 -*-

"""NVE (microcanonical) dynamics in LAMMPS"""

import lammps_step
import logging
import seamm
from seamm_util import ureg, Q_, units_class  # noqa: F401
import seamm_util.printing as printing
from seamm_util.printing import FormattedText as __

logger = logging.getLogger(__name__)
job = printing.getPrinter()
printer = printing.getPrinter('lammps')


class NVE(lammps_step.Energy):

    def __init__(self, flowchart=None, title='NVE dynamics', extension=None):
        """Initialize the node"""

        logger.debug('Creating NVE {}'.format(self))

        super().__init__(flowchart=flowchart, title=title, extension=extension)

        self.description = 'NVE dynamics step in LAMMPS'

        logger.debug("NVE.init() creating NVE_Parameters object")

        self.parameters = lammps_step.NVE_Parameters()

    def description_text(self):
        """Create the text description of what this step will do.
        """

        text = (
            "{time} of microcanonical (NVE) dynamics using a "
            "timestep of {timestep}. The trajectory will be "
            "sampled every {sampling}."
        )

        return text

    def get_input(self, extras=None):
        """Get the input for an NVE dynamics run in LAMMPS"""

        self.description = []
        self.description.append(__(self.header, indent=3 * ' '))

        P = self.parameters.current_values_to_dict(
            context=seamm.flowchart_variables._data
        )

        timestep, P['timestep'] = self.timestep(P['timestep'])

        if extras is not None and 'nsteps' in extras:
            nsteps = extras['nsteps']
        else:
            time = P['time'].to('fs').magnitude
            nsteps = round(time / timestep)

        # Have to fix formatting for printing...
        PP = dict(P)
        for key in PP:
            if isinstance(PP[key], units_class):
                PP[key] = '{:~P}'.format(PP[key])

        self.description.append(
            __(self.description_text(), **PP, indent=7 * ' ')
        )

        # time = lammps_step.to_lammps_units(P['time'], quantity='time')
        # nsteps = round(time / timestep)

        thermo_properties = (
            'time temp press etotal ke pe ebond '
            'eangle edihed eimp evdwl etail ecoul elong'
        )
        properties = 'v_time v_temp v_press v_etotal v_ke v_pe v_emol v_epair'
        title2 = 'tstep t T P Etot Eke Epe Emol Epair'

        lines = []
        nfixes = 0
        lines.append('')
        lines.append('#     NVE dynamics')
        lines.append('')
        lines.append('reset_timestep      0')
        lines.append('timestep            {}'.format(timestep))
        lines.append('thermo_style        custom {}'.format(thermo_properties))
        lines.append('thermo              {}'.format(int(nsteps / 100)))
        nfixes += 1
        lines.append('fix                 {} all nve'.format(nfixes))
        # summary output written 10 times during run so we can see progress
        nevery = 10
        nfreq = int(nsteps / 10)
        nrepeat = int(nfreq / nevery)
        nfreq = nevery * nrepeat
        nfixes += 1
        lines.append(
            'fix                 {} '.format(nfixes) + 'all  ave/time '
            '{} {} {} {} file summary_nve_{}_iter_0.txt'.format(
                nevery, nrepeat, nfreq, properties,
                '_'.join(str(e) for e in self._id)
            )
        )
        # instantaneous output written for averaging
        if P['sampling'] == 'none':
            self.description.append(
                __(
                    "The run will be {nsteps:n} steps of dynamics.",
                    nsteps=nsteps,
                    indent=7 * ' '
                )
            )
        else:
            sampling = lammps_step.to_lammps_units(
                P['sampling'], quantity='time'
            )
            nevery = round(sampling / timestep)
            nfreq = int(nsteps / nevery)
            nrepeat = 1
            nfreq = nevery * nrepeat
            nfixes += 1
            title1 = (
                '!MolSSI trajectory 1.0 LAMMPS, NVE {} steps of {} fs'.format(
                    int(nsteps / nevery), timestep * nevery
                )
            )
            lines.append(
                (
                    "fix                 {} all ave/time {} {} {} {} off 2 "
                    "title1 '{}' title2 '{}' file "
                    "trajectory_nve_{}_iter_0.seamm_trj"
                ).format(
                    nfixes, nevery, nrepeat, nfreq, properties, title1, title2,
                    '_'.join(str(e) for e in self._id)
                )
            )
            self.description.append(
                __(
                    (
                        "The run will be {nsteps:,d} steps of dynamics "
                        "sampled every {nevery:n} steps."
                    ),
                    nsteps=nsteps,
                    nevery=nevery,
                    indent=7 * ' '
                )
            )

        if extras is not None and 'shake' in extras:
            nfixes += 1
            lines.append(extras['shake'].format(nfixes))

        lines.append('')
        lines.append('run                 {}'.format(nsteps))
        lines.append('')

        for fix in range(1, nfixes + 1):
            lines.append('unfix               {}'.format(fix))
        lines.append('')

        return lines

    def timestep(self, value):
        """Get the timestep in the correct units.

        This handles the 'normal', 'accurate' and 'coarse' values,
        which depend on the mass in an empirical fashion.

        Parameters
        ----------
        value : str or Pint quantity
            The desired timestep, which may be a Pint quantity with units
            or one of 'notmal', 'accurate but slow', or 'coarse but fast'

        Returns
        -------
        timestep : float
            The magnitude of the time step in the appropriate LAMMPS units
        """
        masses = self.parent._data['masses']
        min_mass = min(masses)

        # These are based on masses as a proxy for vibrational frequencies
        if min_mass < 10:
            factor = 1
        elif min_mass < 50:
            factor = 2
        else:
            factor = 4

        if value == 'normal':
            timestep = 1.0 * factor
            value = Q_(timestep, ureg.fs)
        elif value == 'accurate but slow':
            timestep = 0.5 * factor
            value = Q_(timestep, ureg.fs)
        elif value == 'coarse but fast':
            timestep = 2.0 * factor
            value = Q_(timestep, ureg.fs)

        timestep = lammps_step.to_lammps_units(value, quantity='time')

        return (timestep, value)
